from typing import Optional
import math

from datetime import datetime, timezone

from sqlalchemy import insert, select, func
from sqlalchemy.ext.asyncio import AsyncConnection
from enum import Enum

from api.db.tables.events import list_events_table


from api.schemas.events.list_events import ListEvent


from api.schemas.endpoints.list_events import all_list_event_adapter, AllListEventResponse


async def get_listevents_page_by_creator_id(
    connection: AsyncConnection, creator_id: int, page: int, limit: int
) -> Optional[AllListEventResponse]:
    """
    Получает список событий заданного создателя по значениям page и limit и creator_id.
    """

    first_event = page * limit - limit
    query = (
        select(
            list_events_table.c.id,
            list_events_table.c.name,
            list_events_table.c.title,
            list_events_table.c.creator_id,
            list_events_table.c.created_at,
            list_events_table.c.schema_,
            list_events_table.c.state,
            list_events_table.c.status,
        )
        .where(list_events_table.c.creator_id == creator_id)  # Фильтрация по creator_id
        .order_by(list_events_table.c.id)
        .offset(first_event)
        .limit(limit)
    )

    count_query = (
        select(func.count())
        .select_from(list_events_table)
        .where(list_events_table.c.creator_id == creator_id)  # Фильтрация по creator_id
    )

    result = await connection.execute(query)
    count_result = await connection.execute(count_query)

    events_data = result.mappings().all()
    total_count = count_result.scalar()
    total_pages = math.ceil(total_count / limit)

    # Здесь предполагается, что all_list_event_adapter.validate_python корректно обрабатывает данные
    validated_list_event = all_list_event_adapter.validate_python(events_data)

    return AllListEventResponse(
        list_event=validated_list_event,
        amount_count=total_count,
        amount_pages=total_pages,
        current_page=page,
        limit=limit,
    )


async def get_listevents_page(connection: AsyncConnection, page, limit) -> Optional[AllListEventResponse]:
    """
    Получает список событий заданного создателя по значениям page и limit.
    """

    first_event = page * limit - (limit)

    query = (
        select(
            list_events_table.c.id,
            list_events_table.c.name,
            list_events_table.c.title,
            list_events_table.c.creator_id,
            list_events_table.c.created_at,
            list_events_table.c.schema,
            list_events_table.c.state,
            list_events_table.c.status,
        )
        .order_by(list_events_table.c.id)
        .offset(first_event)
        .limit(limit)
    )

    count_query = select(func.count()).select_from(list_events_table)

    result = await connection.execute(query)
    count_result = await connection.execute(count_query)

    events_data = result.mappings().all()
    total_count = count_result.scalar()
    total_pages = math.ceil(total_count / limit)

    # Здесь предполагается, что all_list_event_adapter.validate_python корректно обрабатывает данные
    validated_list_event = all_list_event_adapter.validate_python(events_data)

    return AllListEventResponse(
        list_event=validated_list_event,
        amount_count=total_count,
        amount_pages=total_pages,
        current_page=page,
        limit=limit,
    )


async def get_listevents_by_name(connection: AsyncConnection, name: str) -> Optional[ListEvent]:
    """
    Получает list events  по name.
    """
    query = select(list_events_table).where(list_events_table.c.name == name)

    listevents_db_cursor = await connection.execute(query)
    listevents_db = listevents_db_cursor.one_or_none()

    if not listevents_db:
        return None

    listevents_data = {
        column.name: (
            getattr(listevents_db, column.name).name
            if isinstance(getattr(listevents_db, column.name), Enum)
            else getattr(listevents_db, column.name)
        )
        for column in list_events_table.columns
    }

    return ListEvent.model_validate(listevents_data)


async def get_listevents_by_id(connection: AsyncConnection, id: int) -> Optional[ListEvent]:
    """
    Получает listevent по id.
    """
    query = select(list_events_table).where(list_events_table.c.id == id)

    listevents_db_cursor = await connection.execute(query)
    listevents_db = listevents_db_cursor.one_or_none()

    if not listevents_db:
        return None

    listevents_data = {
        column.name: (
            getattr(listevents_db, column.name).name
            if isinstance(getattr(listevents_db, column.name), Enum)
            else getattr(listevents_db, column.name)
        )
        for column in list_events_table.columns
    }

    return ListEvent.model_validate(listevents_data)


async def update_listevents_by_id(connection: AsyncConnection, update_values, listevents):
    """
    Вносит изменеия в нужное поле таблицы list_events_table.
    """
    await connection.execute(
        list_events_table.update().where(list_events_table.c.id == listevents.id).values(**update_values)
    )

    await connection.commit()


async def create_listevents(connection: AsyncConnection, listevents: ListEvent, creator_id: int) -> Optional[ListEvent]:
    """
    Создает нове поле в таблице list_events_table.
    """
    query = insert(list_events_table).values(
        name=listevents.name,
        title=listevents.title,  # добавлено поле title
        creator_id=creator_id,
        created_at=datetime.now(timezone.utc),
        schema=listevents.schema_,  # добавлено поле schema
        state=listevents.state.value,  # добавлено поле state
        status=listevents.status.value,  # добавлено поле status
    )

    await connection.execute(query)

    await connection.commit()

    return listevents
